﻿using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Net;
using System.Reflection;

namespace BootstrapTable.Wenzhixi
{
  /// <summary>
  /// based on http://bootstrap-table.wenzhixin.net.cn/documentation/
  /// </summary>
  public class BootStrapTableColumn
  {
    public String field { get; set; }
    public String title { get; set; }
    public Boolean? sortable { get; set; }
    public Boolean? visible { get; set; }
    public String width { get; set; }
    public String halign { get; set; }
    public String align { get; set; }
    public String formatter { get; set; }
  }


  public class BootStrapTableCreator<ListModel> where ListModel : class
  {
    private IQueryable<PropertyInfo> ListModelProperties { get; set; }
    public Boolean AllowEdit { get; set; }
    public Boolean AllowDelete { get; set; }

    public IList<BootStrapTableColumn> Columns { get; private set; }

    public BootStrapTableCreator()
    {
      ListModelProperties = typeof(ListModel).GetTypeInfo().GetProperties(BindingFlags.Public | BindingFlags.Instance).AsQueryable();

      Columns = CreateColumns();
    }

    private IList<BootStrapTableColumn> CreateColumns()
    {
      var result = new List<BootStrapTableColumn>();

      foreach (var property in ListModelProperties)
      {
        var displayAttrib = property.GetCustomAttribute<DisplayAttribute>();
        var hiddenAttrib = property.GetCustomAttribute<HiddenInputAttribute>();
        var disableSortingAttrib = property.GetCustomAttribute<DisableSortingAttribute>();

        if (displayAttrib == null)
        {
          displayAttrib = new DisplayAttribute()
          {
            Name = property.Name,
            ShortName = property.Name
          };
        }

        var column = new BootStrapTableColumn()
        {
          field = property.Name,
          title = (displayAttrib.ShortName ?? displayAttrib.Name) ?? property.Name,
          sortable = disableSortingAttrib == null,
          visible = hiddenAttrib?.DisplayValue ?? true
        };

        if (property.PropertyType.IsNumericType())
          column.align = "right";

        if (property.PropertyType == typeof(Boolean))
        {
          column.formatter = "checkboxFormatter";
          column.halign = "center";
          column.align = "center";
        }

        result.Add(column);
      }

      return result;
    }

    
    public ContentResult Serialize()
    {     
      if (AllowEdit)
      {
        Columns.Insert(0, new BootStrapTableColumn()
        {
          title = "Edit",
          formatter = "editFormatter",
          halign = "center",
          align = "center",
          width = "40"
        });
      }

      if (AllowDelete)
      {
        Columns.Add(new BootStrapTableColumn()
        {
          title = "Del",
          formatter = "delFormatter",
          halign = "center",
          align = "center",
          width = "40"
        });
      }

      // Get column for title CRUD dialog
      var crudTitleProperty = ListModelProperties.FirstOrDefault(p => p.GetCustomAttribute<CrudTitleAttribute>() != null);

      // Only one field can be key, take the first one found
      var keyProperty = ListModelProperties.FirstOrDefault(p => p.GetCustomAttribute<KeyAttribute>() != null);

      // Only one field for sorting, take the first one found
      var sortProperty = ListModelProperties.FirstOrDefault(p => p.GetCustomAttribute<OrderByAttributeAttributeAttribute>() != null);

      // Get sortdirection
      var sortAttrib = sortProperty?.GetCustomAttribute<OrderByAttributeAttributeAttribute>();

      var settings = new
      {
        CrudTitleFieldName = crudTitleProperty?.Name,
        Pagination = "server",
        UseCache = false,
        Key = keyProperty == null ? "" : keyProperty.Name,
        SortName = sortAttrib == null ? "" :  sortProperty.Name,
        SortOrder = sortAttrib == null ? "" : (sortAttrib.Direction == ListSortDirection.Ascending ? "asc" : "desc"),

        Columns = Columns
      };

      // NullValueHandling must be "ignore" to prevent errors with null value in the bootstrap table
      var content = JsonConvert.SerializeObject(settings, new JsonSerializerSettings() { NullValueHandling = NullValueHandling.Ignore, Formatting = Formatting.Indented });

      return new ContentResult()
      {
        StatusCode = HttpStatusCode.OK.ToInt32(),
        Content = content,
        ContentType = "application/json"
      };
    }
  }
}
