﻿using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Serialization;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Linq.Dynamic.Core;
using System.Net;
using System.Reflection;
using System.Text;

namespace BootstrapTable.Wenzhixi
{
  /// <summary>
  ///  Filter,page and map items for http://bootstrap-table.wenzhixin.net.cn/
  /// </summary>
  public class BootStrapTableData<ListModel> where ListModel : class
  {
    private IQueryable items;
    private String sortName;
    private String sortDirection;
    private Int32 skip;
    private Int32 take;
    private String search;
    private IMapper mapper;

    public Int32 MaxPageSize { get; set; } = 500;

    public BootStrapTableData(IQueryable items, String sort, String order, Int32 skip, Int32 take, String search, IMapper mapper)
    {
      this.items = items;
      this.sortName = sort;
      this.sortDirection = order;
      this.skip = skip;
      this.take = take;
      this.search = search;
      this.mapper = mapper;
    }

    /// <summary>
    /// Valid columns:
    ///   - visible 
    ///   - AutoGenerateFilter!= false
    ///   - No Byte array
    /// </summary>
    /// <param name="recordType"></param>
    /// <returns></returns>
    private IEnumerable<String> ValidSearchFields<T>()
    {
      var ListModelProperties = typeof(T).GetTypeInfo().GetProperties(BindingFlags.Public | BindingFlags.Instance).AsQueryable();

      var nonAutoGenProps = ListModelProperties.Where(p => p.GetCustomAttribute<DisplayAttribute>() != null &&
                                                            p.GetCustomAttribute<DisplayAttribute>().GetAutoGenerateFilter().HasValue).ToList();

      var hiddenInputProps = ListModelProperties.Where(p => p.GetCustomAttribute<HiddenInputAttribute>() != null &&
                                                          p.GetCustomAttribute<HiddenInputAttribute>().DisplayValue == false).ToList();

      var byteArrayTypes = ListModelProperties.Where(p => p.PropertyType == typeof(Byte[])).ToList();

      // Extract invalid types
      var validProperties = ListModelProperties.Except(nonAutoGenProps);
      validProperties = validProperties.Except(hiddenInputProps);
      validProperties = validProperties.Except(byteArrayTypes);

      var result = validProperties.Select(p => p.Name).ToList();

      return result;
    }


    private IQueryable Search(IQueryable items, out Int32 count)
    {
      var itemType = items.ElementType;

      var propertyNames = itemType.GetProperties(BindingFlags.Public | BindingFlags.Instance).Select(p => p.Name).ToList();

      // Apply filtering to all visible column names
      if (!String.IsNullOrEmpty(search))
      {
        var sb = new StringBuilder();

        // create for valid search fields dynamic Linq expression
        foreach (String fieldName in ValidSearchFields<ListModel>())
          sb.AppendFormat("({0} == null ? false : {0}.ToString().IndexOf(@0, @1) >=0) or {1}", fieldName, Environment.NewLine);

        String searchExpression = sb.ToString();
        // remove last "or" occurrence
        searchExpression = searchExpression.Substring(0, searchExpression.LastIndexOf("or"));

        // Apply filtering
        items = items.Where(searchExpression, search, StringComparison.OrdinalIgnoreCase);
      }

      // apply count after filtering
      count = items.Count();

      // Skip requires sorting, so make sure there is always sorting
      String sortExpression = "";

      if (propertyNames.Any(c => c == sortName))
      {
        sortExpression += String.Format("{0} {1}", sortName, sortDirection);
        items = items.OrderBy(sortExpression);
      }

      // save server and client resources
      if (take <= 0)
        take = MaxPageSize;

      items = items.Skip(skip).Take(take);

      return items;
    }


    public IActionResult Serialize()
    {
      // filter and map items
      var mappedItems = mapper.Map<IList<ListModel>>(Search(items, out var count));

      var tableData = new
      {
        // Make sure paging and pagecount is in sync with filtered items
        total = count,
        rows = mappedItems
      };

      // Prepare JSON content
      return new ContentResult()
      {
        StatusCode = HttpStatusCode.OK.ToInt32(),
        Content = JsonConvert.SerializeObject(tableData, new JsonSerializerSettings() { Formatting = Formatting.Indented, ContractResolver = new DefaultContractResolver() }),
        ContentType = "application/json"
      };
    }
  }
}
